close all 
clc 
clear all 

% Paramètres physiques
c = 1.0;         % vitesse d'advection
D = 1.;        % coefficient de diffusion

% Paramètres numériques
L = 1.0;         % longueur du domaine
Nx = 100;        % nombre de points spatiaux
dx = L / (Nx-1); % pas spatial
x = linspace(0, L, Nx);

T = 0.5;         % temps final
CFL= 1.
dt = CFL*min(dx/c,dx^2/(2*D));      % pas de temps respectant la stabilité (stable si alpha<=1/2 et beta <=1)
Nt = round(T/dt);

% Condition initiale : pic gaussien
u = exp(-100*(x - 0.5).^2);
u_new = zeros(size(u));

% Coefficients du schéma
alpha = D * dt / dx^2;
beta  = c * dt / dx;

% Boucle en temps
for n = 1:Nt
    for i = 2:Nx-1
        % Schéma upwind pour l'advection et centré pour la diffusion
        u_new(i) = u(i) ...
                   - beta * (u(i) - u(i-1)) ...
                   + alpha * (u(i+1) - 2*u(i) + u(i-1));
    end

    % Conditions aux limites (Neumann homogènes ici)
    u_new(1) = u_new(2);
    u_new(end) = u_new(end-1);

    % Mise à jour
    u = u_new;

    % Affichage toutes les 1 itérations
    if mod(n,1)==0
        plot(x, u, 'b-', 'LineWidth', 2);
        %axis([0 1 0 1]);
        title(['t = ' num2str(n*dt)]);
        drawnow;
    end
end
